/**---------------------------------------------------------------------------

  Name         :  lcd_EG9018C.h
  Description  :  Header file for EPSON EG9018C passive matrix LCD driver
  Author       :  Aleksander Borysiuk

  History      :  2013.07.16 - ver.1.0

------------------------------------------------------------------------------*/

#ifndef _lcd_EG9018C_H_
#define _lcd_EG9018C_H_

/* Public define -------------------------------------------------------------*/
#define EG9018C_CNTR_GEN_SWD  1         // LCD contrast voltage generator software control
                                        // (0 - disabled, 1 - enabled)

/* Public constants for EPSON EG9018C LCD module -----------------------------*/
#define EG9018C_XRES        640         // LCD horizontal resolution
#define EG9018C_YRES        480         // LCD vertical resolution

#define LCD_PIXEL_OFF         0         // pixel OFF
#define LCD_PIXEL_ON          1         // pixel ON

#define LCD_HORIZONTAL        0         // line draw direction: horizontal
#define LCD_VERTICAL          1         // line draw direction: vertical

#define EG9018C_BKLT_ON      28         // LCD backlight ON (100% PWM)
#define EG9018C_BKLT_OFF      0         // LCD backlight OFF (0% PWM)

#define EG9018C_CNTR_ADJ     85         // LCD contrast (0 - max. to EG9018C_CP1_PERIOD - min.)


/* Public variables ----------------------------------------------------------*/

/* Public functions ----------------------------------------------------------*/
void EG9018C_Init(void);
void LCD_Contrast(uint8_t contrast);
void LCD_Backlight(uint8_t bkltState);
void LCD_PixelOn(uint16_t Xpos, uint16_t Ypos);
void LCD_PixelOff(uint16_t Xpos, uint16_t Ypos);
void LCD_Clear(void);
void LCD_DrawLine(uint16_t Xpos, uint16_t Ypos, uint16_t length, uint8_t direction);
void LCD_DrawRect(uint16_t Xpos, uint16_t Ypos, uint16_t width, uint16_t height);
void LCD_DrawCircle(uint16_t Xpos, uint16_t Ypos, uint16_t radius);
void LCD_DrawPicture(uint16_t Xpos, uint16_t Ypos, uint16_t Xsize, uint16_t Ysize, uint8_t *p_pict);
void LCD_DrawText(uint16_t Xpos, uint16_t Ypos, uint8_t font, uint8_t *p_text);

#endif
/*----------------------------- End of file ----------------------------------*/
