/**-----------------------------------------------------------------------------

  Name         :  main.h
  Description  :  STM32 Butterfly EG9018C demo

  Author       :  Aleksander Borysiuk
  History      :  2013.04.21 - ver.1.0
                  2013.04.29 - STM Peripheral Driver Library upgrade to ver.3.5.0
                  2013.07.18 - EG9018C service functions grouped in lcd_EG9018C module

------------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_conf.h"
#include "stm32_Butterfly.h"
#include "lcd_EG9018C.h"
#include "fonts.h"
#include "pict_logoSTM.h"
#include "pict_STM32.h"

/* Private define ------------------------------------------------------------*/
#define SysCLK_HSE_72MHz 1         // system clock 72MHz
#define IRQ_VectTab_RAM  0         // IRQ vectors in RAM

/* Private macro -------------------------------------------------------------*/
// Access to SRAM via Bit Banding
#define  Var_ResetBit_BB(VarAddr, BitNumber) \
         (*(__IO uint32_t *)(SRAM_BB_BASE | ((VarAddr - SRAM_BASE) << 5) | ((BitNumber) << 2)) = 0)

#define  Var_SetBit_BB(VarAddr, BitNumber)   \
         (*(__IO uint32_t *)(SRAM_BB_BASE | ((VarAddr - SRAM_BASE) << 5) | ((BitNumber) << 2)) = 1)

#define  Var_GetBit_BB(VarAddr, BitNumber)   \
         (*(__IO uint32_t *)(SRAM_BB_BASE | ((VarAddr - SRAM_BASE) << 5) | ((BitNumber) << 2)))

/* Private variables ---------------------------------------------------------*/
GPIO_InitTypeDef GPIO_InitStructure;
EXTI_InitTypeDef EXTI_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;

key_codeTypeDef joystick_key;                                   // joystick key code

uint8_t contrast = EG9018C_CNTR_ADJ;
uint16_t temp;
enum {SCR_0, SCR_1, SCR_2, SCR_3, SCR_END} screen = SCR_3;      // demo screen counter
uint8_t screen_update = 1;                                      // screen update flag

/* Private function prototypes -----------------------------------------------*/
void Delay(__IO uint32_t nCount);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  /* Configure all unused GPIO port pins in Analog Input mode (floating input
     trigger OFF), this will reduce the power consumption and increase the device
     immunity against EMI/EMC *************************************************/

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
                         RCC_APB2Periph_GPIOE, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_Init(GPIOC, &GPIO_InitStructure);
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
                         RCC_APB2Periph_GPIOE, DISABLE);

  // NVIC configuration
#if (IRQ_VectTab_RAM == 1)
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0000);
#else
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0000);
#endif
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  // LED and joystick configuration
  // GPIOE & AFIO clock enable
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE | RCC_APB2Periph_AFIO, ENABLE);

  // LED pins configuration
  GPIO_InitStructure.GPIO_Pin = LED_1 | LED_2;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  // Joystick pins configuration
  GPIO_InitStructure.GPIO_Pin = JOY_OK | JOY_UP | JOY_DOWN | JOY_RIGHT | JOY_LEFT;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING  ;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  // Select joystick pins as EXTI lines
  // Connect EXTI line to PE8 (joystick OK)
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOE, GPIO_PinSource8);
  // Connect EXTI line to PE9 (joystick UP)
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOE, GPIO_PinSource9);
  // Connect EXTI line to PE10 (joystick DOWN)
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOE, GPIO_PinSource10);
  // Connect EXTI line to PE11 (joystick RIGHT)
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOE, GPIO_PinSource11);
  // Connect EXTI line to PE12 (joystick LEFT)
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOE, GPIO_PinSource12);

  // Configure EXTI lines to generate interrupts on falling edge
  EXTI_InitStructure.EXTI_Line = EXTI_Line8 | EXTI_Line9 | EXTI_Line10 | EXTI_Line11 | EXTI_Line12;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  // Register EXTI interrupts
  NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);


  // EG9018C LCD module configuration
  EG9018C_Init();
  LCD_Clear();


  while (1)
  {
  // If joystick has been pressed
  if (joystick_key != KEY_NULL){
    // Service joystick key press
    switch (joystick_key){
      case KEY_OK    :  // Turn off LED1 and LED2
                        GPIO_SetBits(GPIOE, LED_1 | LED_2);

                        // Control LCD backlight
                        if (TIM_GetCapture2(TIM4) == EG9018C_BKLT_OFF){     // if LCD backlight off
                          LCD_Backlight(EG9018C_BKLT_ON);                   // LCD backlight on
                        }
                        else{                                               // if LCD backlight on
                          LCD_Backlight(EG9018C_BKLT_OFF);                  // LCD backlight off
                        }
                        break;

      case KEY_UP    :  // Turn on LED1
                        GPIO_ResetBits(GPIOE, LED_1);

                        // Increase LCD contrast
                        if (contrast > 0){                                  // if contrast is not maximal
                          contrast-=5;                                      // inctrase contrast
                          LCD_Contrast(contrast);                           // update LCD contrasr
                        }
                        break;

      case KEY_DOWN  :  // Turn off LED1
                        GPIO_SetBits(GPIOE, LED_1);

                        // Decrease LCD contrast
                        if (contrast < 160){                                // if contrast is not minimal
                          contrast+=5;                                      // dectrase contrast
                          LCD_Contrast(contrast);                           // update LCD contrasr
                        }
                        break;

      case KEY_RIGHT :  // Turn on LED2
                        GPIO_ResetBits(GPIOE, LED_2);

                        // Show the next demo screen
                        if (screen < SCR_END){
                          screen++;
                          screen_update = 1;
                        }
                        break;

      case KEY_LEFT  :  // Turn off LED2
                        GPIO_SetBits(GPIOE, LED_2);

                        // Show the previous demo screen
                        if (screen > SCR_0){
                          screen--;
                          screen_update = 1;
                        }
                        break;

      default        : //
                       break;
    }

    // Clear key code
    joystick_key = KEY_NULL;
  }


  // Update LCD screen
  if (screen_update){
    screen_update = 0;
    LCD_Clear();       // prepare screen area

    switch (screen){
      case   SCR_0   : // Draw some lines
                       for (temp=0; temp<10; temp++){
                         LCD_DrawLine(0, 48*temp, 55*(temp+1), LCD_HORIZONTAL);
                         LCD_DrawLine(64*temp, 0, 40*(temp+1), LCD_VERTICAL);
                       }
                       break;

      case   SCR_1   : // Draw some rectangles
                       for (temp=0; temp<16; temp++){
                         LCD_DrawRect(10*temp, 10*temp, 30*(temp+1), 15*(temp+1));
                       }
                       break;

      case   SCR_2 :   // Draw some circles
                       for (temp=1; temp<300; temp+=20){
                         LCD_DrawCircle(320, 240, temp);
                       }
                       break;

      case   SCR_3 :   // Draw some pictures
                       LCD_DrawPicture(288, 190, logoSTM_bwWidthPages<<3, logoSTM_bwHeightPixels, (uint8_t*)logoSTM_bwBitmaps);
                       LCD_DrawPicture(240, 250, STM32_bwWidthPages<<3, STM32_bwHeightPixels, (uint8_t*)STM32_bwBitmaps);
                       break;

      case   SCR_END : // Draw some texts
                       LCD_DrawText(30, 50, FONT_FIVE_DOT, (uint8_t*)"Font FIVE DOT / 5x7: EPSON LCD EG9018C test");
                       LCD_DrawText(30, 67, FONT_SIX_DOT, (uint8_t*)"Font SIX DOT / 6x8: EPSON LCD EG9018C test");
                       LCD_DrawText(30, 85, FONT_SEVEN_DOT, (uint8_t*)"Font SEVEN DOT / 7x8: EPSON LCD EG9018C test");
                       LCD_DrawText(30, 103, FONT_NINE_DOT, (uint8_t*)"Font NINE DOT / 9x12: EPSON LCD EG9018C test");
                       LCD_DrawText(30, 125, FONT_TEN_DOT, (uint8_t*)"Font TEN DOT / 10x12: EPSON LCD EG9018C test");
                       LCD_DrawText(30, 147, FONT_FIFTEEN_DOT, (uint8_t*)"Font FIFTEEN DOT / 15x18: EPSON LCD EG9018C test");
                       LCD_DrawText(30, 175, FONT_FIFTEEN_DOT, (uint8_t*)"Font EIGHTEEN DOT:");
                       LCD_DrawText(280, 175, FONT_EIGHTEEN_DOT, (uint8_t*)"012345678");
                       LCD_DrawText(30, 205, FONT_FIFTEEN_DOT, (uint8_t*)"Font MT1:");
                       LCD_DrawText(280, 205, FONT_MT1, (uint8_t*)"012345678");
                       break;

      default        : //
                       screen = SCR_0;
                       break;
    }
  }


    // Insert delay (filters joystick contacts bouncing)
    Delay(0xAFFFF);
  }
}


/**
  * @brief  Inserts a delay time.
  * @param  nCount: specifies the delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
  for(; nCount != 0; nCount--);
}


/**-----------------------------------------------------------------------------
  Name         :  SystemInit
  Description  :  System init for the STM32 Butterfly.
                  Function configures different system clocks.
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void SystemInit(void)
{
  u16 StartUpCounter=0;
  uint32_t HSEStatus;

  /* Setup the microcontroller system. Initialize the Embedded Flash Interface */

  /* Enable HSE=10MHz */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready and if Time out is reached exit */
  do {
    StartUpCounter++;
  } while((RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET) && (StartUpCounter != HSEStartUp_TimeOut));

  if (RCC_GetFlagStatus(RCC_FLAG_HSERDY) == SET){
    HSEStatus = (uint32_t)0x01;
  }
  else {
    HSEStatus = (uint32_t)0x00;
  }

  if (HSEStatus == (uint32_t)0x01){
    /* Enable Prefetch Buffer */
	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
    /* Flash 2 wait state */
	FLASH_SetLatency(FLASH_Latency_2);
    /* HCLK = SYSCLK = 72MHz*/
	RCC_HCLKConfig(RCC_SYSCLK_Div1);
    /* PCLK2 = HCLK = 72MHz*/
	RCC_PCLK2Config(RCC_HCLK_Div1);
    /* PCLK1 = HCLK/2 = 36MHz*/
	RCC_PCLK1Config(RCC_HCLK_Div2);

    /* Configure PLLs ------------------------------------------------------*/
    /* PREDIV2 configuration: PREDIV2CLK = HSE / 2 = 5Hz */
        RCC_PREDIV2Config(RCC_PREDIV2_Div2);

    /* PLL3 configuration: PLL3CLK = (HSE / 2) * 10 = 50 MHz */
	RCC_PLL3Config(RCC_PLL3Mul_10);
    /* Enable PLL3 */
	RCC_PLL3Cmd(ENABLE);
    /* Wait till PLL3 is ready */
        while(RCC_GetFlagStatus(RCC_FLAG_PLL3RDY) == RESET);

    /* MCO configuration: MCO_OUT = PLL3 / 2 = 25MHz for PHY MII*/
    // RCC_MCOConfig(RCC_MCO_PLL3CLK_Div2);
    /* MCO disabled because PA8 is used as TIM1 PWM output*/
    RCC_MCOConfig(RCC_MCO_NoClock);

    /* PLL2 configuration: PLL2CLK = (HSE / 2) * 8 = 40 MHz */
  	RCC_PLL2Config(RCC_PLL2Mul_8);
    /* Enable PLL2 */
	RCC_PLL2Cmd(ENABLE);
    /* Wait till PLL2 is ready */
        while(RCC_GetFlagStatus(RCC_FLAG_PLL2RDY) == RESET);

    /* PREDIV1 configuration: PREDIV1CLK = PLL2 / 5 = 8 MHz */
   	RCC_PREDIV1Config(RCC_PREDIV1_Source_PLL2,RCC_PREDIV1_Div5);

    /* PLL1 configuration: PLLCLK = PREDIV1 * 9 = 72 MHz */
	RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_9);
    /* Enable PLL1 */
        RCC_PLLCmd(ENABLE);
    /* Wait till PLL1 is ready */
        while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

    /* Select PLL as system clock source */
	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
    /* Wait till PLL is used as system clock source */
        while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)0x08);
  }
}


#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
/*----------------------------- End of file ----------------------------------*/
